package org.iphsoft.simon1;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

import org.iphsoft.simon1.AndroidPortAdditions.GameType;
import org.iphsoft.simon1.InGameFacebookActivity.State;
import org.iphsoft.simon1.stats.IStatistics.SettingsInfo;
import org.iphsoft.simon1.stats.StatisticsManager;
import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.util.Appirater;
import org.iphsoft.simon1.util.DialogUtils;
import org.iphsoft.simon1.util.DialogUtils.InputListener;
import org.iphsoft.simon1.util.FileUtils;
import org.iphsoft.simon1.util.IHNMUtils;
import org.iphsoft.simon1.util.ImmersiveModeHelper;
import org.iphsoft.simon1.util.MediaPlayerHelper;
import org.iphsoft.simon1.util.MyLog;
import org.iphsoft.simon1.util.UiThreadUtils;

import android.app.Dialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnTouchListener;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.view.animation.Animation;
import android.view.animation.Animation.AnimationListener;
import android.view.animation.AnimationUtils;
import android.view.inputmethod.InputMethodManager;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.dotemu.ihnmaims.R;
import com.facebook.Response;
import com.facebook.Session;
import com.facebook.SessionState;

public class ScummVMActivity extends FacebookBaseActivity implements ScummVM.OnCompletionListener {

	public final static String EXTRA_STARTING_SAVE_SLOT = "org.iphsoft.simon1.EXTRA_STARTING_SAVE_SLOT";
	public final static String EXTRA_LOAD_OPEN_HOUSE = "org.iphsoft.simon1.EXTRA_LOAD_OPEN_HOUSE";
	public final static String EXTRA_LOAD_FIRST_ROOM = "org.iphsoft.simon1.EXTRA_LOAD_FIRST_ROOM";

	/* Establish whether the hover events are available */
	// private static boolean _hoverAvailable;
	/*
	 * static { try { MouseHelper.checkHoverAvailable(); // this throws
	 * exception if we're // on too old version _hoverAvailable = true; } catch
	 * (Throwable t) { _hoverAvailable = false; } }
	 */

	private class MyScummVM extends ScummVM {
		private boolean usingSmallScreen()
		{
			// Multiple screen sizes came in with Android 1.6. Have
			// to use reflection in order to continue supporting 1.5
			// devices :(
			DisplayMetrics metrics = new DisplayMetrics();
			getWindowManager().getDefaultDisplay().getMetrics(metrics);

			try
			{
				// This 'density' term is very confusing.
				int DENSITY_LOW = metrics.getClass().getField("DENSITY_LOW").getInt(null);
				int densityDpi = metrics.getClass().getField("densityDpi").getInt(metrics);
				return densityDpi <= DENSITY_LOW;
			} catch (Exception e)
			{
				return false;
			}
		}

		public MyScummVM(SurfaceHolder holder)
		{
			super(ScummVMActivity.this.getAssets(), holder);

			// Enable ScummVM zoning on 'small' screens.
			// FIXME make this optional for the user
			// disabled for now since it crops too much
			// enableZoning(usingSmallScreen());
		}

		@Override
		protected void getDPI(float[] values)
		{
			DisplayMetrics metrics = new DisplayMetrics();
			getWindowManager().getDefaultDisplay().getMetrics(metrics);

			values[0] = metrics.xdpi;
			values[1] = metrics.ydpi;
		}

		@Override
		protected void displayMessageOnOSD(final String msg)
		{
			Log.i(LOG_TAG, "OSD: " + msg);

			if (AndroidPortAdditions.SHOW_DEBUG_TOASTS)
			{
				runOnUiThread(new Runnable()
				{

					@Override
					public void run()
					{
						Toast.makeText(ScummVMActivity.this, msg, Toast.LENGTH_SHORT).show();
					}
				});
			}
		}

		@Override
		protected void setWindowCaption(final String caption)
		{
			runOnUiThread(new Runnable()
			{
				public void run()
				{
					setTitle(caption);
				}
			});
		}

		@Override
		protected String[] getPluginDirectories()
		{
			String[] dirs = new String[1];
			dirs[0] = ScummVMApplication.getLastCacheDir().getPath();
			return dirs;
		}

		@Override
		protected void showVirtualKeyboard(final boolean enable)
		{
			runOnUiThread(new Runnable()
			{
				public void run()
				{
					showKeyboard(enable);
				}
			});
		}

		@Override
		protected void onGameOption(final int option)
		{
			runOnUiThread(new Runnable()
			{

				@Override
				public void run()
				{
					switch (option)
					{
					case AndroidPortAdditions.GAME_OPTION_SAVE:
						mSaveLoadFromPostcard = true;
						showSaveDialog();
						break;
					case AndroidPortAdditions.GAME_OPTION_LOAD:
						showLoadDialog();
						break;
					case AndroidPortAdditions.GAME_OPTION_EXIT:
						showExitDialog();
						break;
					}

				}
			});
		}

		@Override
		protected void onGameDisplayStarted()
		{
			// For the Simon 1 hebrew version, we display the splash screen
			// right after
			// game display starts (to cover the vendor logo)
			if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
					&& mLanguagePref.equalsIgnoreCase("he") && mAutoloadSlot == -1)
			{
				// Run on the UI thread after delay
				Handler uiHandler = new Handler(getMainLooper());
				uiHandler.postDelayed(new Runnable()
				{

					@Override
					public void run()
					{
						displaySplashScreenSimon1Hebrew();
					}
				}, SPLASH_SCREEN_START_TIME_HEBREW_VERSION_MILLIS);
			}
		}

		@Override
		protected void gameEventJNIToJava(final int type)
		{

			MyLog.d("ScummVMActivity.MyScummVM: gameEventJNIToJava: " + type);

			runOnUiThread(new Runnable()
			{

				@Override
				public void run()
				{

					switch (type)
					{
					case AndroidPortAdditions.GAME_EVENT_SCALER_FALLBACK:
						// Set the scaling setting to software, and show a
						// toast
						if (AndroidPortAdditions.SUPPORTS_SOFTWARE_SCALER)
						{
							AndroidPortAdditions.instance().setScalingOption(
									AndroidPortAdditions.SCALING_OPTION_SOFT);
						} else
						{
							AndroidPortAdditions.instance().setScalingOption(
									AndroidPortAdditions.SCALING_OPTION_NONE);
						}
						Toast.makeText(ScummVMActivity.this,
								R.string.graphic_setting_fallback_toast, Toast.LENGTH_LONG).show();
						AndroidPortAdditions.instance().setShaderTested(true);

						break;
					case AndroidPortAdditions.GAME_EVENT_SCALER_LQ_FALLBACK:
						// Set the scaling setting to LQ, and show a
						// toast
						AndroidPortAdditions.instance().setScalingOption(
								AndroidPortAdditions.SCALING_OPTION_LQ_SHADER);
						Toast.makeText(ScummVMActivity.this,
								R.string.graphic_setting_fallback_toast, Toast.LENGTH_LONG).show();
						AndroidPortAdditions.instance().setShaderTested(true);
						break;
					case AndroidPortAdditions.GAME_EVENT_SHADER_TEST_SUCCESS:

						// Shader test was successful
						AndroidPortAdditions.instance().setShaderTested(true);
						break;
					case AndroidPortAdditions.GAME_EVENT_SHADER_TEST_FAILURE:

						// Shader test failed - change the setting to
						// software
						// scaling and inform the user

						if (AndroidPortAdditions.SUPPORTS_SOFTWARE_SCALER)
						{
							AndroidPortAdditions.instance().setScalingOption(
									AndroidPortAdditions.SCALING_OPTION_SOFT);
						} else
						{
							AndroidPortAdditions.instance().setScalingOption(
									AndroidPortAdditions.SCALING_OPTION_NONE);
						}
						Toast.makeText(ScummVMActivity.this,
								R.string.graphic_setting_fallback_toast, Toast.LENGTH_LONG).show();

						AndroidPortAdditions.instance().setShaderTested(true);
						break;
					case AndroidPortAdditions.GAME_EVENT_LOAD_SUCCESS:

						if (AndroidPortAdditions.GAME_TYPE != GameType.IHNM)
							hideLoadProgressDialog();

						Toast.makeText(ScummVMActivity.this, R.string.load_success,
								Toast.LENGTH_LONG).show();

						// We can possibly show the Appirater dialog here
						Appirater.appLaunched(ScummVMActivity.this);

						break;
					case AndroidPortAdditions.GAME_EVENT_LOAD_FAILURE:

						hideLoadProgressDialog();

						Toast.makeText(ScummVMActivity.this, R.string.load_failure,
								Toast.LENGTH_LONG).show();

						break;

					case AndroidPortAdditions.GAME_EVENT_SAVE_SUCCESS:

						AndroidPortAdditions.instance().setSaveSlot(mChosenSaveSlot,
								mChosenSaveName);

						// IHNM: delete the old save file from another language
						if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
						{
							String languageSetting = PreferenceManager.getDefaultSharedPreferences(
									ScummVMActivity.this).getString("pref_language", null);

							String oldLanguageSetting = IHNMUtils.getLanguageSettingForSaveSlot(
									mChosenSaveSlot, languageSetting);

							MyLog.d("ScummVMActivity: run: IHNM: slot " + mChosenSaveSlot
									+ " current language " + languageSetting + " old language "
									+ oldLanguageSetting);

							if (oldLanguageSetting != null)
							{
								File oldSave = IHNMUtils.getSaveFileForLanguage(mChosenSaveSlot,
										oldLanguageSetting);
								if (oldSave.exists())
								{
									oldSave.delete();
								}
							}
						}

						// Reset the game time without saving
						mGameTimeWithoutSave = 0;
						mLastResumeTimestamp = 0;

						// SaveReminderHelper.instance().setLastSaveTime(
						// System.currentTimeMillis());

						if (AndroidPortAdditions.SUPPORTS_AUTO_SAVE && mPauseAfterAutoSave)
						{
							mPauseAfterAutoSave = false;

							// Pause the game after auto-save
							if (_scummvm != null)
							{
								_scummvm.setPause(true);
							}

						} else
						{
							// Show save indication to the user if this wasn't
							// an auto-save
							if (AndroidPortAdditions.GAME_TYPE != GameType.IHNM)
								hideSaveProgressDialog();

							if (mHideSaveNotification)
							{
								mHideSaveNotification = false;
							} else
							{
								
								if (AndroidPortAdditions.GAME_TYPE != GameType.IHNM)
								{
									// Show save success toast (disabled for IHNM)
									Toast.makeText(ScummVMActivity.this, R.string.save_success,
											Toast.LENGTH_LONG).show();
								}
							}

							// T7G - play a voice part
							if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
							{
								MediaPlayerHelper.playAudioResource(ScummVMActivity.this,
										R.raw.game_has_been_saved);
							}
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_SAVE_FAILURE:

						if (AndroidPortAdditions.SUPPORTS_AUTO_SAVE && mPauseAfterAutoSave)
						{
							mPauseAfterAutoSave = false;

							// Pause the game after auto-save
							if (_scummvm != null)
							{
								_scummvm.setPause(true);
							}

							if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
							{
								showSaveReminderToast();
							}

						} else
						{
							// Show save indication to the user if this wasn't
							// an auto-save
							hideSaveProgressDialog();

							if (mHideSaveNotification)
							{
								mHideSaveNotification = false;
							} else
							{
								Toast toast = Toast.makeText(ScummVMActivity.this,
										R.string.save_failure, Toast.LENGTH_LONG);
								toast.setGravity(Gravity.CENTER, 0, 0);
								toast.show();
							}
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_HEBREW_TITLE_GONE:
					case AndroidPortAdditions.GAME_EVENT_T7G_INTRO_LOGO_END:
					case AndroidPortAdditions.GAME_EVENT_IHNM_INTRO_LOGO_END:

						// Clear the splash screen
						cleanSplash();

						break;
					case AndroidPortAdditions.GAME_EVENT_FOTAQ_INTRO_LOGO_END:
						UiThreadUtils.getUiHandler().postDelayed(new Runnable()
						{

							@Override
							public void run()
							{
								// Clear the splash screen
								cleanSplash();
							}
						}, 100);
						break;

					case AndroidPortAdditions.GAME_EVENT_POSTCARD_SHOWN:

						showMenu(true);
						break;

					case AndroidPortAdditions.GAME_EVENT_SHOW_MENU:

						showMenu(false);
						break;

					case AndroidPortAdditions.GAME_EVENT_SKIP_INTRO:

						// Relevant for T7G - load the first game save
						if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
						{
							UiThreadUtils.getUiHandler().postDelayed(new Runnable()
							{

								@Override
								public void run()
								{
									restartInFirstRoom();
								}
							}, 1);
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_ENABLE_OPEN_HOUSE:
						// Relevant for T7G - enable open house
						if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
						{
							AndroidPortAdditions.instance().setOpenHouseFlag(true);
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_T7G_CANS_PUZZLE_REACHED:
						// Relevant for T7G - cans puzzle reached
						if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
						{
							if (mLanguagePref.equals("de") || mLanguagePref.equals("fr"))
							{
								showSkipCanPuzzleDialog();
							}
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_T7G_BEDSPREAD_PUZZLE_REACHED:
						// Relevant for T7G - bed puzzle reached
						if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
						{
							if (mLanguagePref.equals("de") || mLanguagePref.equals("fr"))
							{
								showSkipBedPuzzleDialog();
							}
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_T7G_MICROSCOPE_PUZZLE:
						// Relevant for T7G - microscope puzzle reached
						if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
						{
							showMicroscopePuzzlePopup();
						}

						break;
					case AndroidPortAdditions.GAME_EVENT_T7G_BISHOPS_PUZZLE:
						// Relevant for T7G - microscope puzzle reached
						if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
						{
							showBishopsPuzzlePopup();
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_T7G_HEART_PUZZLE_REACHED:
						// Relevant for T7G - bed puzzle reached
						if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
						{
							if (mLanguagePref.equals("de") || mLanguagePref.equals("fr"))
							{
								showSkipHeartPuzzleDialog();
							}
						}

						break;

					case AndroidPortAdditions.GAME_EVENT_SET_FORCE_AUTOSAVING:
						mChosenSaveSlot = AUTO_SAVE_SLOT;
						mChosenSaveName = "Auto save";
						mHideSaveNotification = true;

						break;
					}

				}
			});
		}

		@Override
		protected String[] getSysArchives()
		{
			return new String[0];
		}

		@Override
		protected void showSubtitles(final String text)
		{
			runOnUiThread(new Runnable()
			{

				@Override
				public void run()
				{
					if (mTxtSubtitles != null)
					{
						mTxtSubtitles.setText(text);
						mTxtSubtitles.setVisibility(View.VISIBLE);
					}
				}
			});

		}

		@Override
		protected void hideSubtitles()
		{
			runOnUiThread(new Runnable()
			{

				@Override
				public void run()
				{
					if (mTxtSubtitles != null)
						mTxtSubtitles.setVisibility(View.GONE);
				}
			});
		}

		protected void onAchievementReached(final int achievementId)
		{

			runOnUiThread(new Runnable()
			{

				@Override
				public void run()
				{
					if (!isFacebookLoggedIn())
					{
						// TODO: currently we show the popups all the time
						AndroidPortAdditions.instance().addAchievementId(achievementId);
						showAchievementPopup(achievementId);
					} else if (!AndroidPortAdditions.instance()
							.isAchievementUnlocked(achievementId))
					{
						// If facebook is logged in, just check if we already
						// marked the achievement (which happens only on
						// successful facebook post)
						showAchievementPopup(achievementId);
					}
				}
			});

		}
	}

	private MyScummVM _scummvm;
	private ScummVMEvents _events;
	// private MouseHelper _mouseHelper;
	private Thread _scummvm_thread;

	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		// HACK for quieting down a crash when changing locale
		if (savedInstanceState != null)
		{
			Serializable oldLocaleObject = savedInstanceState.getSerializable("locale");
			if (oldLocaleObject != null)
			{
				Locale oldLocale = (Locale) oldLocaleObject;
				Locale currentLocale = getResources().getConfiguration().locale;
				if (!oldLocale.equals(currentLocale))
				{
					MyLog.w("ScummVMActivity: onCreate: locale change in game activity");
					finish();

					Intent gameMenu = new Intent(this, GameMenuActivity.class);
					gameMenu.putExtra(GameMenuActivity.EXTRA_MID_GAME, false);
					gameMenu.putExtra(GameMenuActivity.EXTRA_FROM_POSTCARD, false);
					startActivity(gameMenu);

					ActivityAnimationUtil.makeActivityNullTransition(this);
					return;
				}
			}
		}

		AndroidPortAdditions.instance().onGameActivityCreate();

		mImmersiveModeHelper = new ImmersiveModeHelper(this);

		/*
		 * // Turn off soft buttons on Android 4.0+ try { Field
		 * lowProfileFlagField =
		 * View.class.getField("SYSTEM_UI_FLAG_LOW_PROFILE"); if
		 * (lowProfileFlagField != null) { int lowProfileFlag =
		 * (Integer)lowProfileFlagField.get(View.class); Method
		 * setSystemUiVisibility =
		 * getWindow().getDecorView().getClass().getMethod
		 * ("setSystemUiVisibility", Integer.TYPE); if (setSystemUiVisibility !=
		 * null) { setSystemUiVisibility.invoke(getWindow().getDecorView(),
		 * lowProfileFlag); } } } catch (Exception e) { // TODO Auto-generated
		 * catch block e.printStackTrace(); }
		 */

		getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

		setVolumeControlStream(AudioManager.STREAM_MUSIC);

		setContentView(R.layout.main);
		takeKeyEvents(true);

		// This is a common enough error that we should warn about it
		// explicitly.
		if (!Environment.getExternalStorageDirectory().canRead()
				|| !Environment.getExternalStorageDirectory().canWrite())
		{

			DialogUtils.createNegativePositiveDialog(this, R.string.quit, 0, R.string.no_sdcard,
					new DialogInterface.OnClickListener()
					{
						public void onClick(DialogInterface dialog, int which)
						{
							finish();
							ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);

						}
					}, null).show();

			return;
		}

		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);

		mTxtSubtitles = (TextView) findViewById(R.id.txtSubtitles);
		if (mTxtSubtitles != null)
			mTxtSubtitles.setVisibility(View.GONE);

		mFrameSplash = findViewById(R.id.frameSplash);
		mFrameSplash.setOnTouchListener(new OnTouchListener()
		{
			@Override
			public boolean onTouch(View v, MotionEvent event)
			{
				return true;
			}
		});
		
		mImgSplash = (ImageView)findViewById(R.id.imgSplash);
		mImgSplash.setImageResource(R.drawable.developer_logo);
		//BitmapUtils.setScaledBackground(this, mImgSplash, R.drawable.developer_logo);

		SurfaceView main_surface = (SurfaceView) findViewById(R.id.main_surface);

		main_surface.requestFocus();

		getFilesDir().mkdirs();

		// Store savegames on external storage if we can, which means they're
		// world-readable and don't get deleted on uninstall.
		String savePath = Environment.getExternalStorageDirectory() + "/"
				+ AndroidPortAdditions.instance().getAppName() + "/Saves/";
		File saveDir = new File(savePath);
		saveDir.mkdirs();
		if (!saveDir.isDirectory())
		{
			// If it doesn't work, show an error and quit
			Toast.makeText(this, R.string.no_sdcard, Toast.LENGTH_LONG);
			finish();
			ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);

			return;
		}

		// Start ScummVM
		_scummvm = new MyScummVM(main_surface.getHolder());

		// Receive callback for ScummVM completion
		_scummvm.setOnCompletionListener(this);

		ArrayList<String> args = new ArrayList<String>();

		MyLog.d("ScummVMActivity: onCreate: game path: "
				+ AndroidPortAdditions.instance().getGameFileDir());

		args.add("ScummVM");
		args.add("--config=" + getFileStreamPath("scummvmrc").getPath());
		args.add("--path=" + AndroidPortAdditions.instance().getGameFileDir());
		args.add("--gui-theme=scummmodern");
		args.add("--savepath=" + savePath);

		// Edit command line args according to user selection
		// music mode
		String musicMode = prefs.getString("pref_music_mode", "enhanced");
		if (musicMode.equals("enhanced"))
		{
			args.add("--use-music=0");
		} else if (musicMode.equals("original_mt32"))
		{
			args.add("--music-driver=mt32");
			args.add("--use-music=1");

		} else if (musicMode.equals("original_eas"))
		{
			args.add("--music-driver=eas");
			args.add("--use-music=1");

		} else if (musicMode.equals("original"))
		{
			args.add("--use-music=1");
		} else if (musicMode.equals("none"))
		{
			args.add("--use-music=2");
		}

		// Volume settings
		int musicVolume = prefs.getInt("pref_music_volume", 192);
		int speechVolume = prefs.getInt("pref_speech_volume", 192);
		int sfxVolume = prefs.getInt("pref_sfx_volume", 192);
		args.add("--music-volume=" + musicVolume);
		args.add("--speech-volume=" + speechVolume);

		if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
		{
			// In FOTAQ, the engine ignores the speech volume and just uses the
			// SFX volume setting - so we use it here.
			args.add("--sfx-volume=" + speechVolume);
		} else
		{
			args.add("--sfx-volume=" + sfxVolume);
		}

		// subtitles
		String voiceMode = prefs.getString("pref_voice_mode", "voice_subtitles");
		if (voiceMode.equals("voice_subtitles"))
		{
			args.add("--subtitles");
		} else if (voiceMode.equals("subtitles"))
		{
			args.add("--subtitles");
			args.add("--speech_mute");
		} else if (voiceMode.equals("voice"))
		{
			if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM
					|| AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
			{
				args.add("--no-subtitles");
			}
		}

		if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ
				|| AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			// Subtitle speed
			args.add("--talkspeed=" + AndroidPortAdditions.instance().getSubtitleSpeed());
		}

		// Language option
		mLanguagePref = prefs.getString("pref_language", null);
		String languageOption = "en";
		if (mLanguagePref != null)
		{
			if (mLanguagePref.equals("de"))
				languageOption = "de";
			else if (mLanguagePref.equals("he"))
				languageOption = "hb";
			else if (mLanguagePref.equals("es"))
				languageOption = "es";
			else if (mLanguagePref.equals("fr"))
				languageOption = "fr";
			else if (mLanguagePref.equals("it"))
				languageOption = "it";
			else if (mLanguagePref.equals("cz"))
				languageOption = "cz";
			else if (mLanguagePref.equals("pl"))
				languageOption = "pl";
			else if (mLanguagePref.equals("ru"))
				languageOption = "ru";
		}

		// Set the ScummVM language argument
		args.add("--language=" + languageOption);

		// Set the "alt intro" flag if needed
		if (AndroidPortAdditions.instance().getStartNewGameAltIntro())
			args.add("--alt-intro");

		// Set the slot to auto-load if there is
		mAutoloadSlot = getIntent().getIntExtra(EXTRA_STARTING_SAVE_SLOT, -1);

		// If we start in game beginning after intro, set the load slot to our
		// hard-coded save file
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G
				&& getIntent().getBooleanExtra(EXTRA_LOAD_FIRST_ROOM, false))
		{
			mAutoloadSlot = T7G_HARDCODED_SAVE_SLOT;

			File afterIntroSave = new File(savePath + T7G_HARDCODED_SAVE);
			if (afterIntroSave.exists())
			{
				afterIntroSave.delete();
			}

			FileUtils.copyAssetToExternalStorage(this, T7G_GAME_START_SAVE, AndroidPortAdditions
					.instance().getAppName() + "/Saves/" + T7G_HARDCODED_SAVE);
		}

		// If we start in open house mode, set the load slot to our hard-coded
		// open house save file, and make sure it exists
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G
				&& getIntent().getBooleanExtra(EXTRA_LOAD_OPEN_HOUSE, false))
		{
			mAutoloadSlot = T7G_HARDCODED_SAVE_SLOT;
			_scummvm.gameEventJavaToJNI(AndroidPortAdditions.GAME_EVENT_START_GAME_IN_OPEN_HOUSE);

			File openHouseSave = new File(savePath + T7G_HARDCODED_SAVE);
			if (openHouseSave.exists())
			{
				openHouseSave.delete();
			}

			FileUtils.copyAssetToExternalStorage(this, T7G_OPEN_HOUSE_SAVE, AndroidPortAdditions
					.instance().getAppName() + "/Saves/" + T7G_HARDCODED_SAVE);
		}

		MyLog.d("ScummVMActivity: onCreate: autoload slot: " + mAutoloadSlot);

		if (mAutoloadSlot != -1
				&& (AndroidPortAdditions.GAME_TYPE == GameType.T7G
						|| AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ || AndroidPortAdditions.GAME_TYPE == GameType.IHNM))
		{
			MyLog.d("ScummVMActivity: onCreate: using normal ScummVM load function" + "");
			// For T7G or FOTAQ, just use the normal ScummVM load function
			args.add("--save-slot=" + mAutoloadSlot);
			_scummvm.setAutoLoadSlot(-1);
		} else
		{
			MyLog.d("ScummVMActivity: onCreate: using customized load function for slot: "
					+ mAutoloadSlot);

			_scummvm.setAutoLoadSlot(mAutoloadSlot);
		}

		if (mAutoloadSlot != -1
				&& (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
						|| AndroidPortAdditions.GAME_TYPE == GameType.SIMON2 || AndroidPortAdditions.GAME_TYPE == GameType.IHNM))
		{
			showLoadProgressDialog();
		} else
		{
			// We can possibly show the Appirater dialog here
			Appirater.appLaunched(this);
		}

		// Set the game string
		switch (AndroidPortAdditions.GAME_TYPE)
		{
		case SIMON1:
			args.add("simon1");
			break;
		case SIMON2:
			args.add("simon2");
			break;
		case FOTAQ:
			args.add("queen");
			break;
		case INDY_FOA:
			args.add("atlantis");
			break;
		case T7G:
			args.add("t7g");
			break;
		case IHNM:
			args.add("ihnm");
			break;
		}

		String[] argsArray = args.toArray(new String[args.size()]);

		MyLog.d("ScummVMActivity: onCreate: args: " + Arrays.toString(argsArray));

		_scummvm.setArgs(argsArray);

		_events = new ScummVMEvents(this, _scummvm);

		main_surface.setOnKeyListener(_events);
		main_surface.setOnTouchListener(_events);

		_scummvm_thread = new Thread(_scummvm, "ScummVM");

		if (mAutoloadSlot != -1)
		{
			_scummvm_thread.start();
			cleanSplash();
		} else if ((AndroidPortAdditions.GAME_TYPE == GameType.SIMON1 && mLanguagePref
				.equalsIgnoreCase("he")))
		{
			// For Simon 1 hebrew version, start the ScummVM thread
			// normally
			_scummvm_thread.start();
		} else if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			// For T7G, start the ScummVM thread normally and show the splash
			_scummvm_thread.start();
			displaySplashScreenT7G();
		} else if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
		{
			// For FOTAQ, show the splash for a while, then start the thread,
			// then wait until the logo end
			displaySplashScreenFOTAQ();
		} else if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			// For IHNM, start the ScummVM thread normally and show the splash
			_scummvm_thread.start();
			displaySplashScreenIHNM();
		} else
		{
			// For other versions, start it only after the splash screen
			displaySplashScreenDefaultBehavior();
		}

		// Send game start statistics with all settings info
		SettingsInfo settingInfo = new SettingsInfo();
		settingInfo.mGraphicModeType = AndroidPortAdditions.instance().getAutoGraphicMode() ? "auto"
				: "manual";
		settingInfo.mGraphicMode = prefs.getString("pref_scaler_option", "unknown");
		settingInfo.mVoiceMode = voiceMode;
		settingInfo.mMusicMode = musicMode;
		settingInfo.mControlMode = prefs.getString("pref_control_mode", "unknown");
		settingInfo.mLanguage = mLanguagePref;

		// If we're logged in to Facebook, sync achievements
		if (isFacebookLoggedIn())
		{
			requestRefreshPermissions();
			requestCurrentAchievements();
		}

		StatisticsManager.getStatistics().reportGameStart(settingInfo);
	}

	@Override
	protected void onStart()
	{
		super.onStart();

		StatisticsManager.getStatistics().reportActivityStart(this);
	}

	@Override
	protected void onStop()
	{
		super.onStop();

		StatisticsManager.getStatistics().reportActivityStop(this);
	}

	@Override
	public void onWindowFocusChanged(boolean hasFocus)
	{
		// TODO Auto-generated method stub
		super.onWindowFocusChanged(hasFocus);

		if (hasFocus && mImmersiveModeHelper != null)
		{
			mImmersiveModeHelper.supportFullScreenImmersiveMode();
		}
	}

	@Override
	public void onResume()
	{
		Log.d(ScummVM.LOG_TAG, "onResume");

		super.onResume();

		if (_scummvm != null)
		{
			// Prevent any race condition where the auto-saving happened only
			// after resuming
			mPauseAfterAutoSave = false;

			_scummvm.setPause(false);
			mPauseState = false;

			// We do this to work around Android's behavior of not calling any
			// surface callbacks
			// if the screen is locked.
			_scummvm.forceReinitSurfaceIfNeeded();

			// Update the touch mode according to settings
			SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
			String controlMode = prefs.getString("pref_control_mode", "control_mode_touch");
			if (controlMode.equals("control_mode_touch"))
			{
				_scummvm.setTouchpadMode(false);
				_scummvm.gameEventJavaToJNI(AndroidPortAdditions.GAME_EVENT_SHOW_NEW_ACTION_BAR);
			} else if (controlMode.equals("control_mode_classic"))
			{
				_scummvm.setTouchpadMode(true);
				_scummvm.gameEventJavaToJNI(AndroidPortAdditions.GAME_EVENT_HIDE_NEW_ACTION_BAR);

			} else if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2
					&& controlMode.equals("control_mode_touch_classic_bar"))
			{
				_scummvm.setTouchpadMode(false);
				_scummvm.gameEventJavaToJNI(AndroidPortAdditions.GAME_EVENT_HIDE_NEW_ACTION_BAR);

			} else
			{
				MyLog.e("ScummVMActivity: onPause: unknown control mode: " + controlMode);
			}
		}

		// SaveReminderHelper.instance().setGamePaused(false);

		mLastResumeTimestamp = System.currentTimeMillis();
	}

	@Override
	public void onPause()
	{
		Log.d(ScummVM.LOG_TAG, "onPause");

		super.onPause();

		if (_scummvm != null)
		{

			if (AndroidPortAdditions.SUPPORTS_AUTO_SAVE && !(mImgSplash.getVisibility() == View.VISIBLE)
					&& !(AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ && mExiting))
			{
				// For games supporting auto-save, attempt saving and then
				// pause.
				// Do not allow auto saving when exiting FOTAQ.
				autoSaveAndPause();
			} else
			{
				// For other games, just pause
				_scummvm.setPause(true);
			}
		}

		if (mLastResumeTimestamp != 0)
		{
			mGameTimeWithoutSave += (System.currentTimeMillis() - mLastResumeTimestamp);
		}

		MyLog.d("ScummVMActivity: onPause: game time without save " + mGameTimeWithoutSave);

		// Display save reminder if pausing this activity and a certain game
		// time has passed without saving (for games without auto-save)
		if (!AndroidPortAdditions.SUPPORTS_AUTO_SAVE
				&& mGameTimeWithoutSave >= MIN_GAP_FOR_SAVE_REMINDER)
		{
			showSaveReminderToast();
		}

		// SaveReminderHelper.instance().setGamePaused(true);
		// SaveReminderHelper.instance().setLastPauseTime(System.currentTimeMillis());
	}

	private void showSaveReminderToast()
	{
		String appName = AndroidPortAdditions.instance().getAppName();
		String message = ScummVMActivity.this.getString(R.string.save_game_reminder, appName);

		Toast.makeText(ScummVMActivity.this, message, Toast.LENGTH_LONG).show();
	}

	private void autoSaveAndPause()
	{
		MyLog.d("ScummVMActivity: autoSaveAndPause: ");

		mPauseAfterAutoSave = true;
		mChosenSaveSlot = AUTO_SAVE_SLOT;
		mChosenSaveName = "Auto save";
		_scummvm.saveGame(AUTO_SAVE_SLOT, false);
	}

	@Override
	public void onDestroy()
	{
		Log.d(ScummVM.LOG_TAG, "onDestroy");

		AndroidPortAdditions.instance().onGameActivityDestroy();

		super.onDestroy();

		if (_events != null && _scummvm != null)
		{
			MyLog.d("ScummVMActivity: onDestroy: sending quit event");
			_scummvm.setPause(false);
			_scummvm.setOnCompletionListener(null);
			_events.sendQuitEvent();
		}

		MyLog.d("ScummVMActivity: onDestroy: end");

	}

	@Override
	public void onSaveInstanceState(Bundle outState)
	{
		MyLog.d("ScummVMActivity: onSaveInstanceState: ");

		Locale currentLocale = getResources().getConfiguration().locale;
		outState.putSerializable("locale", currentLocale);
		super.onSaveInstanceState(outState);
	}

	@Override
	public boolean onTrackballEvent(MotionEvent e)
	{
		if (_events != null)
			return _events.onTrackballEvent(e);

		return false;
	}

	private void showKeyboard(boolean show)
	{
		SurfaceView main_surface = (SurfaceView) findViewById(R.id.main_surface);
		InputMethodManager imm = (InputMethodManager) getSystemService(INPUT_METHOD_SERVICE);

		if (show)
			imm.showSoftInput(main_surface, InputMethodManager.SHOW_IMPLICIT);
		else
			imm.hideSoftInputFromWindow(main_surface.getWindowToken(),
					InputMethodManager.HIDE_IMPLICIT_ONLY);
	}

	/*
	 * @Override public boolean onCreateOptionsMenu(Menu menu) { MenuInflater
	 * inflater = getMenuInflater(); inflater.inflate(R.menu.main_menu, menu);
	 * return true; }
	 */
	/*
	 * @Override public boolean onOptionsItemSelected(MenuItem item) { switch
	 * (item.getItemId()) { case R.id.settings: startActivityForResult(new
	 * Intent(this, SettingsActivity.class), SETTINGS_ACTIVITY_REQUEST_CODE);
	 * return true; case R.id.help: startActivity(new Intent(this,
	 * HelpActivity.class)); return true; case R.id.save: showSaveDialog();
	 * return true; case R.id.load: showLoadDialog(); return true; default:
	 * return super.onOptionsItemSelected(item); } }
	 */

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		MyLog.d("ScummVMActivity: onActivityResult: instance: " + this);

		if (DialogUtils.callFromOnActivityResult(requestCode, resultCode, data))
		{
			return;
		}

		// Restart if needed (settings changed)
		if (requestCode == GAME_MENU_REQUEST_CODE)
		{
			int loadSlot = -1;

			if (data != null)
			{
				loadSlot = data.getIntExtra(GameMenuActivity.EXTRA_SAVE_SLOT, -1);
			}

			switch (resultCode)
			{
			case GameMenuActivity.RESULT_REQUEST_RESTART:
				MyLog.d("ScummVMActivity: onActivityResult: RESULT_REQUEST_RESTART");
				restart(-1);
				return;
			case GameMenuActivity.RESULT_REQUEST_SAVE:
				mSaveLoadFromPostcard = false;

				showSaveDialog();
				return;
			case GameMenuActivity.RESULT_REQUEST_QUIT:
				// We show a confirmation dialog before exiting
				showExitDialog();
				return;
			case GameMenuActivity.RESULT_REQUEST_LOAD:

				if (_scummvm.checkLoadConditions())
				{
					// If we can load mid-game, do it
					_scummvm.loadGame(loadSlot);
				} else
				{
					// If not, restart
					restart(loadSlot);
				}

				return;
			case GameMenuActivity.RESULT_REQUEST_RESTART_LOAD:
				MyLog.d("ScummVMActivity: onActivityResult: RESULT_REQUEST_RESTART_LOAD");
				restart(loadSlot);
				return;

			case GameMenuActivity.RESULT_REQUEST_OPEN_HOUSE:
				restartInOpenHouse();
				return;
			}
		} else if (requestCode == FACEBOOK_ACTIVITY_REQUEST_CODE)
		{
			// Get the facebook activity state
			if (data != null)
			{
				int value = data.getIntExtra(InGameFacebookActivity.EXTRA_RESULT, -1);
				InGameFacebookActivity.State state = State.fromValue(value);

				MyLog.d("ScummVMActivity: onActivityResult: Facebook activity state: "
						+ state.toString());

				// If the permissions were accepted, post the last achievement
				if (state == State.PERMISSIONS_GRANTED)
				{
					showAchievementShareDialog(mLastUnlockedAchievement);
				}

				StatisticsManager.getStatistics().reportInGameFacebookDialogState(state);
			}

			// Reset the last unlocked achievement
			mLastUnlockedAchievement = -1;
		}

		super.onActivityResult(requestCode, resultCode, data);
	}

	@Override
	public boolean onKeyUp(int keyCode, KeyEvent event)
	{

		if (keyCode == KeyEvent.KEYCODE_BACK)
		{
			// We show a confirmation dialog before exiting
			showExitDialog();
			return true;
		} else if (keyCode == KeyEvent.KEYCODE_MENU)
		{
			if (AndroidPortAdditions.TEST_ACHIEVEMENTS)
			{
				// Just a quick way to test the achievement popup
				showAchievementPopup(1);
			} else if (AndroidPortAdditions.MENU_FORCE_PAUSE)
			{
				if (_scummvm != null)
				{
					mPauseState = !mPauseState;
					_scummvm.setPause(mPauseState);
					if (mPauseState == false)
						_scummvm.forceReinitSurfaceIfNeeded();
				}

			} else
			{
				// Show the game menu
				showMenu(false);
			}

			return true;
		}

		return super.onKeyUp(keyCode, event);
	}

	private void showExitDialog()
	{
		OnClickListener onClick = new OnClickListener()
		{

			@Override
			public void onClick(DialogInterface arg0, int arg1)
			{
				if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
				{
					// "Come back!"
					MediaPlayerHelper.playAudioResource(ScummVMActivity.this, R.raw.come_back);
				}

				if (_scummvm != null)
					_scummvm.setPause(false);

				exit();
				finish();
				ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);

			}
		};

		if (AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ)
		{
			DialogUtils.showPopupDialogActivity(this, R.string.no, R.string.yes,
					R.string.exit_confirmation_message, null, onClick);
		} else
		{
			DialogUtils.showNegativePositiveDialogOrActivity(this, R.string.no, R.string.yes,
					R.string.exit_confirmation_message, null, onClick, true);
		}
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			ActivityAnimationUtil.makeActivityFadeTransition(this);
		}
	}

	private void showMenu(boolean fromPostcard)
	{
		Intent gameMenu = new Intent(this, GameMenuActivity.class);
		gameMenu.putExtra(GameMenuActivity.EXTRA_MID_GAME, true);
		gameMenu.putExtra(GameMenuActivity.EXTRA_FROM_POSTCARD, fromPostcard);
		gameMenu.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
		startActivityForResult(gameMenu, GAME_MENU_REQUEST_CODE);

		ActivityAnimationUtil.makeActivityFadeTransition(this);
	}

	private void showSaveDialog()
	{

		List<String> allSaveSlots = AndroidPortAdditions.instance().getSaveSlots();

		// If there's an auto-save, don't show the first save slot - it is
		// reserved for auto-saving
		if (AndroidPortAdditions.SUPPORTS_AUTO_SAVE)
		{
			allSaveSlots.remove(0);
		}

		mSaveSlots = allSaveSlots.toArray(new String[0]);

		int negText = AndroidPortAdditions.GAME_TYPE == GameType.IHNM ? R.string.back
				: R.string.cancel;
		DialogUtils.showListDialog(this, negText, R.string.save_game, R.string.choose_save_slot,
				new OnClickListener()
				{

					@Override
					public void onClick(DialogInterface dlg, int which)
					{
						// Convert to slot number
						mChosenSaveSlot = which + 1;
						if (AndroidPortAdditions.SUPPORTS_AUTO_SAVE)
						{
							// Increment the slot index, since the list didn't
							// include the auto-save slot
							++mChosenSaveSlot;
						}

						// Proceed to edit save name
						showSaveNameDialog();

						dlg.dismiss();
					}
				}, mSaveSlots, true);
	}

	private void showLoadDialog()
	{

		// Generate a list of save slots to display
		List<String> allSaveSlots = AndroidPortAdditions.instance().getSaveSlots();
		mSaveSlotIndexes = AndroidPortAdditions.instance().getSaveSlotsIndexes();
		if (mSaveSlotIndexes.size() == 0)
		{
			// Nothing to load
			Toast.makeText(this, R.string.no_saved_games, Toast.LENGTH_LONG).show();
			return;
		}
		List<String> displayedSaveSlots = new ArrayList<String>();

		for (Integer i : mSaveSlotIndexes)
		{
			displayedSaveSlots.add(allSaveSlots.get(i));
		}

		DialogUtils.showListDialog(this, R.string.cancel, R.string.load_game,
				R.string.choose_load_slot, new OnClickListener()
				{

					@Override
					public void onClick(DialogInterface dlg, int which)
					{
						// Load the game (convert index to save slot)
						_scummvm.loadGame(mSaveSlotIndexes.get(which) + 1);
						dlg.dismiss();
					}
				}, displayedSaveSlots.toArray(new String[0]), true);
	}

	private void showSaveNameDialog()
	{
		// Setup the initial input, depending on whether the save slot is
		// existing or new
		String initialInput;
		List<Integer> indexes = AndroidPortAdditions.instance().getSaveSlotsIndexes();

		// Calculate the slot 0-based index from the list choice (based on
		// auto-save slot availability)
		int slotNameIndex = mChosenSaveSlot - 1;

		if (indexes.contains(slotNameIndex))
		{
			if (AndroidPortAdditions.SUPPORTS_AUTO_SAVE)
			{
				// Subtract 1 because we removed auto-save from the string list
				--slotNameIndex;
			}

			// Show the current save name
			initialInput = mSaveSlots[slotNameIndex];
		} else
		{

			initialInput = "";

			// Show time and date
			// initialInput = DateFormat.format("MMM dd yyyy h:mmaa",
			// System.currentTimeMillis())
			// .toString();
		}

		Dialog dlg = DialogUtils.createNegativePositiveDialog(this, R.string.cancel, R.string.ok,
				R.string.enter_save_name, new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface dialog, int whichButton)
					{
						// Canceled.
						dialog.dismiss();
					}
				}, null, new InputListener()
				{
					public void onInputReceived(String input)
					{
						if (input.equals("BADGER"))
						{
							// Easter egg - enable open house
							AndroidPortAdditions.instance().setOpenHouseFlag(true);
						}
						
						if (input.equals(""))
						{
							// Default save name
							input = "NO NAME";
						}

						mChosenSaveName = input.toUpperCase();
						showSaveProgressDialog();
						_scummvm.saveGame(mChosenSaveSlot, mSaveLoadFromPostcard);
					}
				}, initialInput, false);

		dlg.show();
	}

	private void showSkipCanPuzzleDialog()
	{
		DialogUtils.showPopupDialogActivity(this, R.string.no, R.string.yes,
				R.string.skip_english_puzzle_popup, null, new OnClickListener()
				{

					@Override
					public void onClick(DialogInterface dialog, int which)
					{
						_scummvm.gameEventJavaToJNI(AndroidPortAdditions.GAME_EVENT_SKIP_CAN_PUZZLE);
					}
				});
	}

	private void showSkipBedPuzzleDialog()
	{
		Dialog dlg = DialogUtils.createNegativePositiveDialog(this, R.string.no, R.string.yes,
				R.string.skip_english_puzzle_popup, null, new OnClickListener()
				{

					@Override
					public void onClick(DialogInterface dialog, int which)
					{
						_scummvm.gameEventJavaToJNI(AndroidPortAdditions.GAME_EVENT_SKIP_BEDSPREAD_PUZZLE);
					}
				});

		dlg.show();
	}

	private void showSkipHeartPuzzleDialog()
	{
		DialogUtils.showPopupDialogActivity(this, R.string.no, R.string.yes,
				R.string.skip_english_puzzle_popup, null, new OnClickListener()
				{

					@Override
					public void onClick(DialogInterface dialog, int which)
					{
						_scummvm.gameEventJavaToJNI(AndroidPortAdditions.GAME_EVENT_SKIP_HEART_PUZZLE);
					}
				});
	}

	private void showMicroscopePuzzlePopup()
	{
		DialogUtils.showPopupDialogActivity(this, 0, R.string.ok, R.string.microscope_puzzle_popup,
				null, null);
	}

	private void showBishopsPuzzlePopup()
	{
		DialogUtils.showPopupDialogActivity(this, 0, R.string.ok, R.string.bishops_puzzle_popup,
				null, null);
	}

	private void showAchievementShareDialog(final int achievementId)
	{
		DialogUtils.showInputActivity(this, R.string.cancel, R.string.ok,
				R.string.facebook_share_achievement_dialog_title,
				new DialogInterface.OnClickListener()
				{
					public void onClick(DialogInterface dialog, int whichButton)
					{
						// Canceled.
						dialog.dismiss();
					}
				}, null, new InputListener()
				{
					public void onInputReceived(String input)
					{
						postNewAchievement(achievementId, true, input);
					}
				});
	}

	private void showSaveProgressDialog()
	{
		mSaveProgressDialog = DialogUtils.createProgressDialog(this, R.string.save_progress_dialog);
		mSaveProgressDialog.show();

		// In IHNM, hide the save dialog after a delay
		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			UiThreadUtils.getUiHandler().postDelayed(new Runnable()
			{

				@Override
				public void run()
				{
					hideSaveProgressDialog();
				}
			}, 1000);
		}
	}

	private void hideSaveProgressDialog()
	{
		if (mSaveProgressDialog != null && mSaveProgressDialog.isShowing())
		{
			mSaveProgressDialog.dismiss();
		}
	}

	private void showLoadProgressDialog()
	{
		mLoadProgressDialog = DialogUtils.createProgressDialog(this, R.string.load_progress_dialog);
		mLoadProgressDialog.show();

		// In IHNM, hide the load dialog after a delay
		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			UiThreadUtils.getUiHandler().postDelayed(new Runnable()
			{

				@Override
				public void run()
				{
					hideLoadProgressDialog();
				}
			}, 1000);
		}
	}

	private void hideLoadProgressDialog()
	{
		if (mLoadProgressDialog != null && mLoadProgressDialog.isShowing())
		{
			mLoadProgressDialog.dismiss();
		}
	}

	private void showAchievementPopup(final int achievementId)
	{
		// Don't show the popup if the setting is off
		if (!AndroidPortAdditions.instance().getEnablePopups())
		{
			return;
		}

		// Inflate the popup layout in the top area
		final ViewGroup achievementPopup = (ViewGroup) findViewById(R.id.achievementPopup);
		achievementPopup.removeAllViews();
		getLayoutInflater().inflate(R.layout.achievement_popup, achievementPopup);

		// Fill the view with achievement details
		ImageView imgIcon = (ImageView) achievementPopup.findViewById(R.id.imgIcon);
		Drawable iconDrawable = getResources().obtainTypedArray(R.array.achievement_icons)
				.getDrawable(achievementId);
		imgIcon.setImageDrawable(iconDrawable);

		TextView txtDesc = (TextView) achievementPopup.findViewById(R.id.txtDescription);
		txtDesc.setText(getResources().obtainTypedArray(R.array.achievement_names).getString(
				achievementId));

		mFacebookShareClicked = false;

		final View btnClose = achievementPopup.findViewById(R.id.btnClose);
		final View btnShare = achievementPopup.findViewById(R.id.btnShare);

		// Set the button listener
		btnShare.setOnClickListener(new View.OnClickListener()
		{
			@Override
			public void onClick(View v)
			{
				mFacebookShareClicked = true;

				mLastUnlockedAchievement = achievementId;

				if (isFacebookLoggedIn()
						&& getPublishPermissionState() == PublishPermissionState.ACCEPTED)
				{
					showAchievementShareDialog(achievementId);
				} else
				{
					startActivityForResult(new Intent(ScummVMActivity.this,
							InGameFacebookActivity.class), FACEBOOK_ACTIVITY_REQUEST_CODE);
					ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);
				}

				StatisticsManager.getStatistics().reportAchievementShareClick(achievementId);
			}
		});

		// Setup a slide-in, slide-out animation for the popup
		Animation slideIn = AnimationUtils.loadAnimation(this, R.anim.achievement_slide_in);
		final Animation slideOut = AnimationUtils.loadAnimation(this, R.anim.achievement_slide_out);

		final Runnable closePopup = new Runnable()
		{

			@Override
			public void run()
			{
				btnShare.setOnClickListener(null);
				btnClose.setOnClickListener(null);
				achievementPopup.startAnimation(slideOut);
				UiThreadUtils.getUiHandler().removeCallbacks(this);
			}
		};

		// Start a timer after the 1st animation ends, then start the 2nd
		// animation
		slideIn.setAnimationListener(new AnimationListener()
		{
			@Override
			public void onAnimationStart(Animation animation)
			{
			}

			@Override
			public void onAnimationRepeat(Animation animation)
			{
			}

			@Override
			public void onAnimationEnd(Animation animation)
			{
				UiThreadUtils.getUiHandler().postDelayed(closePopup, ACHIEVEMENT_POPUP_TIME_MILLIS);
			}
		});

		// Make sure the view is not visible after sliding out
		slideOut.setAnimationListener(new AnimationListener()
		{

			@Override
			public void onAnimationStart(Animation animation)
			{
			}

			@Override
			public void onAnimationRepeat(Animation animation)
			{
			}

			@Override
			public void onAnimationEnd(Animation animation)
			{
				achievementPopup.removeAllViews();
				achievementPopup.setVisibility(View.GONE);

				// If the "share" button was not clicked, and we're logged in
				// with permissions, post an implicit share
				if (isFacebookLoggedIn() && !mFacebookShareClicked)
				{
					PublishPermissionState publishPermissionsState = getPublishPermissionState();
					if (publishPermissionsState == PublishPermissionState.ACCEPTED)
					{
						postNewAchievement(achievementId, false, null);
					}
				}
			}
		});

		// Setup the close button
		btnClose.setOnClickListener(new View.OnClickListener()
		{
			@Override
			public void onClick(View v)
			{
				closePopup.run();
			}
		});

		achievementPopup.startAnimation(slideIn);
	}

	private void exit()
	{
		MyLog.d("ScummVMActivity: exit: ");

		mExiting = true;

		if (_scummvm != null)
		{
			_scummvm.setOnCompletionListener(null);
		}

		if (_events != null)
		{
			_events.sendQuitEvent();
			try
			{
				MyLog.d("ScummVMActivity: exit: joining");
				_scummvm_thread.join();
			} catch (InterruptedException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		_scummvm = null;

		mLastResumeTimestamp = 0;
		mGameTimeWithoutSave = 0;

		// SaveReminderHelper.instance().stopReminders();
	}

	private void restart(int loadSlot)
	{
		MyLog.d("ScummVMActivity: restart: " + loadSlot);

		if (_scummvm != null)
		{
			_scummvm.setRestarting();
			_scummvm.setPause(false);
		}

		exit();
		finish();

		Intent loadIntent = new Intent(this, PrepareGameFilesActivity.class);
		if (loadSlot != -1)
		{
			loadIntent.putExtra(ScummVMActivity.EXTRA_STARTING_SAVE_SLOT, loadSlot);
		}
		startActivity(loadIntent);
		ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);

	}

	private void restartInOpenHouse()
	{
		MyLog.d("ScummVMActivity: restartInOpenHouse: ");

		if (_scummvm != null)
		{
			_scummvm.setRestarting();
			_scummvm.setPause(false);
		}

		exit();
		finish();

		Intent openHouseIntent = new Intent(this, PrepareGameFilesActivity.class);
		openHouseIntent.putExtra(ScummVMActivity.EXTRA_LOAD_OPEN_HOUSE, true);
		startActivity(openHouseIntent);
		ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);
	}

	private void restartInFirstRoom()
	{
		MyLog.d("ScummVMActivity: restartInFirstRoom: ");

		if (_scummvm != null)
		{
			_scummvm.setRestarting();
			_scummvm.setPause(false);
		}

		exit();
		finish();

		Intent restartIntent = new Intent(this, PrepareGameFilesActivity.class);
		restartIntent.putExtra(ScummVMActivity.EXTRA_LOAD_FIRST_ROOM, true);
		startActivity(restartIntent);
		ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);

	}

	private void displaySplashScreenDefaultBehavior()
	{
		MyLog.d("ScummVMActivity: displaySplashScreenDefaultBehavior: ");

		mFrameSplash.setVisibility(View.VISIBLE);
		mImgSplash.setVisibility(View.VISIBLE);

		new Handler().postDelayed(new Runnable()
		{

			@Override
			public void run()
			{
				_scummvm_thread.start();
				cleanSplash();

			}
		}, SPLASH_SCREEN_TIME_MILLIS);
	}

	private void displaySplashScreenSimon1Hebrew()
	{
		MyLog.d("ScummVMActivity: displaySplashScreenSimon1Hebrew: ");

		mFrameSplash.setVisibility(View.VISIBLE);
		mImgSplash.setVisibility(View.VISIBLE);
	}

	private void displaySplashScreenT7G()
	{
		MyLog.d("ScummVMActivity: displaySplashScreentT7G: ");

		mFrameSplash.setVisibility(View.VISIBLE);
		mImgSplash.setVisibility(View.VISIBLE);

		new Handler().postDelayed(new Runnable()
		{

			@Override
			public void run()
			{
				// Switch to the 2nd developer logo
				mImgSplash.setImageResource(R.drawable.developer_logo_2);
			}
		}, T7G_SPLASH_SCREEN_SWITCH_TIME_MILLIS);
	}

	private void displaySplashScreenIHNM()
	{
		MyLog.d("ScummVMActivity: displaySplashScreentIHNM: ");

		mFrameSplash.setVisibility(View.VISIBLE);
		mImgSplash.setVisibility(View.VISIBLE);
		
		new Handler().postDelayed(new Runnable()
		{

			@Override
			public void run()
			{
				// Switch to the 2nd developer logo
				mImgSplash.setImageResource(R.drawable.developer_logo_2);
			}
		}, IHNM_SPLASH_SCREEN_SWITCH_TIME_MILLIS);
	}

	private void displaySplashScreenFOTAQ()
	{
		MyLog.d("ScummVMActivity: displaySplashScreentFOTAQ: ");

		if (AndroidPortAdditions.HIDE_GAME_START_LOGO)
		{
			mImgSplash.setVisibility(View.GONE);
		} else
		{
			mFrameSplash.setVisibility(View.VISIBLE);
			mImgSplash.setVisibility(View.VISIBLE);
		}

		new Handler().postDelayed(new Runnable()
		{

			@Override
			public void run()
			{
				_scummvm_thread.start();
			}
		}, SPLASH_SCREEN_TIME_MILLIS_FOTAQ);
	}

	private void cleanSplash()
	{
		mFrameSplash.setVisibility(View.GONE);
		mImgSplash.setVisibility(View.GONE);
		mImgSplash.setImageDrawable(null);
		mImgSplash.setBackgroundDrawable(null);
	}

	/**
	 * Called when ScummVM finished running
	 */
	@Override
	public void onComplete()
	{

		MyLog.d("ScummVMActivity: onComplete: ");

		runOnUiThread(new Runnable()
		{

			@Override
			public void run()
			{
				if (!mExiting)
				{
					// If we didn't intentionally exit, the game ended - show
					// the game menu
					exit();
					finish();
					startActivity(new Intent(ScummVMActivity.this, GameMenuActivity.class)
							.putExtra(GameMenuActivity.EXTRA_MID_GAME, false));
					ActivityAnimationUtil.makeActivityFadeTransition(ScummVMActivity.this);
				}
			}
		});
	}

	@Override
	protected void onSessionStateChange(Session session, SessionState state, Exception exception)
	{
		if (state.isOpened())
		{
			MyLog.d("ScummVMActivity: Facebook logged in...");
		} else if (state.isClosed())
		{
			MyLog.d("ScummVMActivity: Facebook logged out...");
		}
	}

	@Override
	protected void onLoggedInStateChange(boolean loggedIn)
	{
		if (loggedIn)
		{
			AndroidPortAdditions.instance().clearAllAchievements();
			requestCurrentAchievements();
		} else
		{
			AndroidPortAdditions.instance().clearAllAchievements();
		}
	}

	@Override
	protected void onPublishPermissionsResponse(boolean granted)
	{
	}

	@Override
	protected void onAchievementPosted(int index, boolean explicitShare, Response response,
			String shareMessage)
	{
		MyLog.d("ScummVMActivity: onAchievementPosted: index: " + index + " response: " + response);

		// Verify the response
		if (verifyPublishResponse(response))
		{
			MyLog.d("ScummVMActivity: onAchievementPosted: achievement post is successful");

			if (explicitShare)
			{
				// Add achievement locally
				AndroidPortAdditions.instance().addAchievementId(index);

				Toast.makeText(ScummVMActivity.this,
						R.string.facebook_share_achievement_success_toast, Toast.LENGTH_SHORT)
						.show();
			}

			StatisticsManager.getStatistics().reportSuccessfulAchievementPost(index, shareMessage);
		} else if (explicitShare)
		{
			Toast.makeText(ScummVMActivity.this, R.string.facebook_share_achievement_failure_toast,
					Toast.LENGTH_SHORT).show();
		}
	}

	@Override
	protected void onCurrentAchievementsResponse(List<Integer> ids)
	{
		AndroidPortAdditions.instance().setAchievementIds(ids);
	}

	private String[] mSaveSlots;
	private List<Integer> mSaveSlotIndexes;

	private int mChosenSaveSlot = 0;
	private String mChosenSaveName = "";
	private boolean mHideSaveNotification = false;

	private ImageView mImgSplash;
	private View mFrameSplash;


	private String mLanguagePref;

	private int mAutoloadSlot = -1;

	private int mLastUnlockedAchievement = -1;

	// Track whether the share button was clicked for the last achievement
	private boolean mFacebookShareClicked = false;

	private Dialog mSaveProgressDialog;
	private Dialog mLoadProgressDialog;

	private boolean mSaveLoadFromPostcard = false;

	private boolean mPauseAfterAutoSave = false;

	private long mGameTimeWithoutSave = 0;
	private long mLastResumeTimestamp = 0;

	private boolean mExiting = false;

	private ImmersiveModeHelper mImmersiveModeHelper;

	private TextView mTxtSubtitles;

	private boolean mPauseState;

	private static final int GAME_MENU_REQUEST_CODE = 1123;
	private static final int FACEBOOK_ACTIVITY_REQUEST_CODE = 4312;

	private static final int DIALOG_ID_EXIT = 1;

	private static final long SPLASH_SCREEN_TIME_MILLIS = 3000;
	private static final long SPLASH_SCREEN_TIME_MILLIS_FOTAQ = 2000;

	private static final long T7G_SPLASH_SCREEN_SWITCH_TIME_MILLIS = 8000;
	private static final long IHNM_SPLASH_SCREEN_SWITCH_TIME_MILLIS = 10000;


	private static final long SPLASH_SCREEN_TIME_HEBREW_VERSION_MILLIS = 6000;
	private static final long SPLASH_SCREEN_START_TIME_HEBREW_VERSION_MILLIS = 0;

	private static final long ACHIEVEMENT_POPUP_TIME_MILLIS = 5000;

	public static final long MIN_GAP_FOR_SAVE_REMINDER = 5 * 60 * 1000;

	public static final String T7G_HARDCODED_SAVE = "t7g.009";
	public static final int T7G_HARDCODED_SAVE_SLOT = 9;

	public static final String T7G_GAME_START_SAVE = "t7g.009";
	public static final String T7G_OPEN_HOUSE_SAVE = "t7g.010";

	public static final int AUTO_SAVE_SLOT = 1;
}
