package org.iphsoft.simon1.util;

import org.iphsoft.simon1.AndroidPortAdditions;
import org.iphsoft.simon1.GameMenuActivity;
import org.iphsoft.simon1.T7GSphinxDialogActivity;
import org.iphsoft.simon1.T7GTranslucentDialogActivity;
import org.iphsoft.simon1.AndroidPortAdditions.GameType;
import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.ui.IHNMListChooser;

import android.app.Activity;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.Intent;
import android.graphics.drawable.ColorDrawable;
import android.sax.StartElementListener;
import android.text.InputFilter;
import android.text.InputType;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup.LayoutParams;
import android.view.Window;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.dotemu.ihnmaims.R;

/**
 * Utils for simplifying dialog creation
 * 
 * @author omergilad
 */
public class DialogUtils {

	public static interface InputListener {
		public void onInputReceived(String input);
	}

	public static Dialog createNegativePositiveDialog(Context context, int negative, int positive,
			int message, final OnClickListener negativeListener,
			final OnClickListener positiveListener)
	{

		return createNegativePositiveDialog(context, negative, positive, message, negativeListener,
				positiveListener, null, "", false);
	}

	public static Dialog createNegativePositiveDialog(Context context, int negative, int positive,
			int message, final OnClickListener negativeListener,
			final OnClickListener positiveListener, final InputListener inputListener,
			String initialInput, boolean multilineText)
	{

		String negText = negative != 0 ? context.getString(negative) : null;
		String posText = positive != 0 ? context.getString(positive) : null;
		String msgText = message != 0 ? context.getString(message) : null;

		return createNegativePositiveDialog(context, negText, posText, msgText, negativeListener,
				positiveListener, inputListener, initialInput, multilineText);
	}

	public static Dialog createNegativePositiveDialog(Context context, String negative,
			String positive, String message, final OnClickListener negativeListener,
			final OnClickListener positiveListener, final InputListener inputListener,
			String initialInput, boolean multilineText)
	{

		Dialog dlg = new Dialog(context);
		Window window = dlg.getWindow();

		window.setLayout(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
		window.setGravity(Gravity.CENTER);

		window.requestFeature(Window.FEATURE_NO_TITLE);
		ImmersiveModeHelper.supportFullScreenImmersiveModeForDialog(dlg);

		window.setBackgroundDrawable(new ColorDrawable(0x00000000));

		dlg.setContentView(R.layout.game_dialog);

		TextView txtTitle = (TextView) dlg.findViewById(R.id.txtTitle);
		if (message != null)
			txtTitle.setText(message);

		dlg.findViewById(R.id.imgSpinner).setVisibility(View.GONE);
		dlg.findViewById(R.id.lvList).setVisibility(View.GONE);
		// dlg.findViewById(R.id.separator1).setVisibility(View.GONE);
		// dlg.findViewById(R.id.separator2).setVisibility(View.GONE);

		final EditText etInput = (EditText) dlg.findViewById(R.id.etInput);
		
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			etInput.setFilters(new InputFilter[] {new InputFilter.AllCaps()});
		}
		
		if (multilineText)
		{
			etInput.setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_CAP_SENTENCES
					| InputType.TYPE_TEXT_FLAG_MULTI_LINE
					| InputType.TYPE_TEXT_VARIATION_LONG_MESSAGE);
			etInput.setSingleLine(false);
			etInput.setLines(3);

		} else
		{
			etInput.setLines(1);
			etInput.setMaxLines(1);
			etInput.setSingleLine(true);
		}

		etInput.setText(initialInput);
		if (inputListener == null)
		{
			// No need for input
			etInput.setVisibility(View.GONE);
		}

		TextView btnNegative = (TextView) dlg.findViewById(R.id.btnNegative);
		TextView btnPositive = (TextView) dlg.findViewById(R.id.btnPositive);

		if (negative != null)
		{
			btnNegative.setText(negative);
			btnNegative.setTag(dlg);
			btnNegative.setOnClickListener(new View.OnClickListener()
			{

				@Override
				public void onClick(View btn)
				{

					DialogInterface dlgInterface = (DialogInterface) btn.getTag();

					if (negativeListener != null)
					{
						negativeListener.onClick(dlgInterface, dlgInterface.BUTTON_NEGATIVE);
					}

					dlgInterface.dismiss();
				}
			});
		} else
		{
			btnNegative.setVisibility(View.GONE);
			dlg.findViewById(R.id.fillerView1).setVisibility(View.GONE);
		}

		if (positive != null)
		{
			btnPositive.setText(positive);
			btnPositive.setTag(dlg);
			btnPositive.setOnClickListener(new View.OnClickListener()
			{

				@Override
				public void onClick(View btn)
				{
					DialogInterface dlgInterface = (DialogInterface) btn.getTag();

					if (positiveListener != null)
					{
						positiveListener.onClick(dlgInterface, dlgInterface.BUTTON_POSITIVE);
					}

					if (inputListener != null)
					{
						inputListener.onInputReceived(etInput.getText().toString());
					}

					dlgInterface.dismiss();
				}
			});
		} else
		{
			btnPositive.setVisibility(View.GONE);
			dlg.findViewById(R.id.fillerView1).setVisibility(View.GONE);

			// Remove the negative button margin
			LinearLayout.LayoutParams lp = (LinearLayout.LayoutParams) btnNegative
					.getLayoutParams();
			lp.leftMargin = 0;
			btnNegative.setLayoutParams(lp);
		}

		return dlg;
	}

	public static Dialog createProgressDialog(Context context, int message)
	{

		Dialog dlg = new Dialog(context);
		Window window = dlg.getWindow();

		window.setLayout(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
		window.setGravity(Gravity.CENTER);

		window.requestFeature(Window.FEATURE_NO_TITLE);
		ImmersiveModeHelper.supportFullScreenImmersiveModeForDialog(dlg);

		window.setBackgroundDrawable(new ColorDrawable(0x00000000));

		dlg.setContentView(R.layout.game_dialog);

		TextView txtTitle = (TextView) dlg.findViewById(R.id.txtTitle);
		txtTitle.setText(message);

		View imgSpinner = dlg.findViewById(R.id.imgSpinner);
		Animation rotation = AnimationUtils.loadAnimation(context, R.anim.rotate);
		imgSpinner.startAnimation(rotation);

		dlg.findViewById(R.id.etInput).setVisibility(View.GONE);
		dlg.findViewById(R.id.btnPositive).setVisibility(View.GONE);
		dlg.findViewById(R.id.btnNegative).setVisibility(View.GONE);
		dlg.findViewById(R.id.lvList).setVisibility(View.GONE);
		dlg.findViewById(R.id.layoutButtons).setVisibility(View.GONE);
		// dlg.findViewById(R.id.separator1).setVisibility(View.GONE);
		// dlg.findViewById(R.id.separator2).setVisibility(View.GONE);

		return dlg;
	}

	public static void showListDialog(Activity context, int negative, int positive, int message,
			final OnClickListener selectionListener, CharSequence[] items, boolean midGame)
	{
		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			sCurrentSelectionListener = selectionListener;
			
			// Start the list activity
			Intent intent = new Intent(context, IHNMListChooser.class);
			intent.putExtra(GameMenuActivity.EXTRA_MID_GAME, midGame);
			intent.putExtra(IHNMListChooser.EXTRA_NEGATIVE_BUTTON, context.getString(negative));
			intent.putExtra(IHNMListChooser.EXTRA_POSITIVE_BUTTON, context.getString(positive));
			intent.putExtra(IHNMListChooser.EXTRA_TITLE, context.getString(message));
			intent.putExtra(IHNMListChooser.EXTRA_LIST_ITEMS, items);

			context.startActivityForResult(intent, DIALOG_ACTIVITY_REQUEST_CODE);		
			ActivityAnimationUtil.makeActivityNullTransition(context);

			
			return;
		}

		Dialog dlg = new Dialog(context);
		Window window = dlg.getWindow();

		window.setLayout(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
		window.setGravity(Gravity.CENTER);

		window.requestFeature(Window.FEATURE_NO_TITLE);
		ImmersiveModeHelper.supportFullScreenImmersiveModeForDialog(dlg);

		window.setBackgroundDrawable(new ColorDrawable(0x00000000));

		dlg.setContentView(R.layout.game_dialog);

		TextView txtTitle = (TextView) dlg.findViewById(R.id.txtTitle);
		txtTitle.setText(message);

		ListView lvList = (ListView) dlg.findViewById(R.id.lvList);

		// Adding header to make the ListView display an additional divider on
		// top
		View invisible1 = new View(context);
		invisible1.setLayoutParams(new AbsListView.LayoutParams(0, 0));
		lvList.addHeaderView(invisible1);

		lvList.setTag(dlg);
		lvList.setAdapter(new ListDialogAdapter(context, items));
		lvList.setOnItemClickListener(new OnItemClickListener()
		{

			@Override
			public void onItemClick(AdapterView<?> view, View arg1, int pos, long arg3)
			{
				DialogInterface dlgInterface = (DialogInterface) view.getTag();
				dlgInterface.dismiss();

				if (selectionListener != null)
				{
					// Subtracting 1 because of the header view
					selectionListener.onClick(dlgInterface, pos - 1);
				}
			}
		});

		dlg.findViewById(R.id.imgSpinner).setVisibility(View.GONE);
		dlg.findViewById(R.id.etInput).setVisibility(View.GONE);
		dlg.findViewById(R.id.btnPositive).setVisibility(View.GONE);

		TextView btnNegative = (TextView) dlg.findViewById(R.id.btnNegative);

		if (negative != 0)
		{
			btnNegative.setText(negative);
			btnNegative.setTag(dlg);
			btnNegative.setOnClickListener(new View.OnClickListener()
			{

				@Override
				public void onClick(View btn)
				{

					DialogInterface dlgInterface = (DialogInterface) btn.getTag();
					dlgInterface.dismiss();
				}
			});
		} else
		{
			btnNegative.setVisibility(View.GONE);
		}

		// Remove the negative button margin
		LinearLayout.LayoutParams lp = (LinearLayout.LayoutParams) btnNegative.getLayoutParams();
		lp.leftMargin = 0;
		btnNegative.setLayoutParams(lp);

		dlg.findViewById(R.id.fillerView1).setVisibility(View.GONE);

		dlg.show();
	}

	/**
	 * Wrapper for calls that show a popup OR an activity, depending on game
	 * type
	 */
	public static void showNegativePositiveDialogOrActivity(Activity context, int negative,
			int positive, int message, OnClickListener negativeListener,
			OnClickListener positiveListener, boolean midGame)
	{
		switch (AndroidPortAdditions.GAME_TYPE)
		{
		case SIMON1:
		case SIMON2:
		case FOTAQ:
		case IHNM:
			Dialog dlg = createNegativePositiveDialog(context, negative, positive, message,
					negativeListener, positiveListener);
			dlg.show();
			break;
		case T7G:
			showNegativePositiveActivity(context, negative, positive, message, negativeListener,
					positiveListener, midGame);
			break;
		}
	}

	public static void showNegativePositiveActivity(Activity context, int negative, int positive,
			int message, OnClickListener negativeListener, OnClickListener positiveListener,
			boolean midGame)
	{

		// Persist the listener, so we can invoke them in onActivityResult
		sCurrentPositiveListener = positiveListener;
		sCurrentNegativeListener = negativeListener;

		// Set up the intent
		String negText = negative != 0 ? context.getString(negative) : null;
		String posText = positive != 0 ? context.getString(positive) : null;
		String msgText = message != 0 ? context.getString(message) : null;

		Intent dialogActivityIntent = new Intent(context, T7GSphinxDialogActivity.class);
		dialogActivityIntent.putExtra(T7GSphinxDialogActivity.EXTRA_MESSAGE, msgText);
		dialogActivityIntent.putExtra(T7GSphinxDialogActivity.EXTRA_POSITIVE_BUTTON, posText);
		dialogActivityIntent.putExtra(T7GSphinxDialogActivity.EXTRA_NEGATIVE_BUTTON, negText);
		dialogActivityIntent.putExtra(GameMenuActivity.EXTRA_MID_GAME, midGame);

		// Launch the dialog activity
		context.startActivityForResult(dialogActivityIntent, DIALOG_ACTIVITY_REQUEST_CODE);
	}

	public static void showInputActivity(Activity context, int negative, int positive, int message,
			OnClickListener negativeListener, OnClickListener positiveListener,
			InputListener inputListener)
	{
		// Persist the listener, so we can invoke them in onActivityResult
		sCurrentPositiveListener = positiveListener;
		sCurrentNegativeListener = negativeListener;
		sCurrentInputListener = inputListener;

		// Set up the intent
		Intent dialogActivityIntent = new Intent(context, T7GTranslucentDialogActivity.class);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_MESSAGE, message);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_POSITIVE_BUTTON, positive);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_NEGATIVE_BUTTON, negative);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_SHOW_INPUT, true);

		// Launch the dialog activity
		context.startActivityForResult(dialogActivityIntent, DIALOG_ACTIVITY_REQUEST_CODE);

		ActivityAnimationUtil.makeActivityFadeTransition(context);
	}

	public static void showPopupDialogActivity(Activity context, int negative, int positive,
			int message, OnClickListener negativeListener, OnClickListener positiveListener)
	{
		// Persist the listener, so we can invoke them in onActivityResult
		sCurrentPositiveListener = positiveListener;
		sCurrentNegativeListener = negativeListener;

		// Set up the intent
		Intent dialogActivityIntent = new Intent(context, T7GTranslucentDialogActivity.class);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_MESSAGE, message);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_POSITIVE_BUTTON, positive);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_NEGATIVE_BUTTON, negative);
		dialogActivityIntent.putExtra(T7GTranslucentDialogActivity.EXTRA_SHOW_INPUT, false);

		// Launch the dialog activity
		context.startActivityForResult(dialogActivityIntent, DIALOG_ACTIVITY_REQUEST_CODE);

		ActivityAnimationUtil.makeActivityFadeTransition(context);
	}

	public static boolean callFromOnActivityResult(int requestCode, int resultCode, Intent data)
	{
		if (requestCode == DIALOG_ACTIVITY_REQUEST_CODE)
		{
			DialogInterface dummyDlgInterface = new DialogInterface()
			{
				@Override
				public void dismiss()
				{
				}

				@Override
				public void cancel()
				{
				}
			};

			if (resultCode == Activity.RESULT_OK)
			{
				if (sCurrentPositiveListener != null)
				{
					sCurrentPositiveListener.onClick(dummyDlgInterface,
							DialogInterface.BUTTON_POSITIVE);
				}
				if (sCurrentInputListener != null && data != null)
				{
					String input = data
							.getStringExtra(T7GTranslucentDialogActivity.EXTRA_RESULT_INPUT);
					sCurrentInputListener.onInputReceived(input);
				}
				if (sCurrentSelectionListener != null && data != null)
				{
					int index = data
							.getIntExtra(IHNMListChooser.EXTRA_RESULT, 0);
					sCurrentSelectionListener.onClick(dummyDlgInterface, index);
				}
			} else if (resultCode == Activity.RESULT_CANCELED && sCurrentNegativeListener != null)
			{
				sCurrentNegativeListener
						.onClick(dummyDlgInterface, DialogInterface.BUTTON_NEGATIVE);
			}

			sCurrentPositiveListener = null;
			sCurrentNegativeListener = null;
			sCurrentInputListener = null;
			sCurrentSelectionListener = null;

			return true;
		}

		return false;
	}

	private static class ListDialogAdapter extends ArrayAdapter<CharSequence> {

		public ListDialogAdapter(Context context, CharSequence[] objects)
		{
			super(context, R.layout.dialog_list_row, R.id.txtValue, objects);
		}

	}

	private static OnClickListener sCurrentPositiveListener = null;
	private static OnClickListener sCurrentSelectionListener = null;
	private static OnClickListener sCurrentNegativeListener = null;
	private static InputListener sCurrentInputListener = null;

	private static int DIALOG_ACTIVITY_REQUEST_CODE = 3454;
}
